@@ -4,7 +4,6 @@
 
 #![forbid(unsafe_code)]
 
-// #![allow(unused_imports)]
 const SQLITE_U64_ERROR: &str = r##"SQLite cannot natively store unsigned 64-bit integers, so Turbosql does not support u64 fields. Use i64, u32, f64, or a string or binary format instead. (see https://github.com/trevyn/turbosql/issues/3 )"##;
 
 use once_cell::sync::Lazy;
@@ -14,7 +13,6 @@ use quote::{format_ident, quote, ToTokens};
 use rusqlite::{params, Connection, Statement};
 use serde::{Deserialize, Serialize};
 use std::collections::BTreeMap;
-use std::sync::Mutex;
 use syn::parse::{Parse, ParseStream};
 use syn::punctuated::Punctuated;
 use syn::spanned::Spanned;
@@ -31,16 +29,6 @@ const MIGRATIONS_FILENAME: &str = "test.migrations.toml";
 mod insert;
 mod update;
 
-// trait Ok<T> {
-//  fn ok(self) -> Result<T, anyhow::Error>;
-// }
-
-// impl<T> Ok<T> for Option<T> {
-//  fn ok(self) -> Result<T, anyhow::Error> {
-//   self.ok_or_else(|| anyhow::anyhow!("NoneError"))
-//  }
-// }
-
 #[derive(Debug, Clone)]
 struct Table {
  ident: Ident,
@@ -78,12 +66,8 @@ struct MiniColumn {
  sql_type: String,
 }
 
-// static TEST_DB: Lazy<Mutex<Connection>> =
-//  Lazy::new(|| Mutex::new(Connection::open_in_memory().unwrap()));
-
-static LAST_TABLE_NAME: Lazy<Mutex<String>> = Lazy::new(|| Mutex::new("none".to_string()));
-
-static TABLES: Lazy<Mutex<BTreeMap<String, MiniTable>>> = Lazy::new(|| Mutex::new(BTreeMap::new()));
+static U8_ARRAY_RE: Lazy<regex::Regex> =
+ Lazy::new(|| regex::Regex::new(r"^Option < \[u8 ; \d+\] >$").unwrap());
 
 // #[proc_macro]
 // pub fn set_db_path(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
@@ -114,24 +98,6 @@ struct ExecuteTokens {
  tokens: proc_macro2::TokenStream,
 }
 
-#[derive(Debug)]
-struct QueryParams {
- params: Punctuated<Expr, Token![,]>,
-}
-
-impl Parse for QueryParams {
- fn parse(input: ParseStream) -> syn::Result<Self> {
-  Ok(QueryParams {
-   params: if input.peek(Token![,]) {
-    input.parse::<Token![,]>().unwrap();
-    input.parse_terminated(Expr::parse)?
-   } else {
-    Punctuated::new()
-   },
-  })
- }
-}
-
 #[derive(Clone, Debug)]
 struct ResultType {
  container: Option<Ident>,
@@ -148,18 +114,18 @@ struct ResultType {
 
 #[derive(Debug)]
 struct MembersAndCasters {
- members: Vec<(Ident, Ident, usize)>,
- struct_members: Vec<proc_macro2::TokenStream>,
+ //  members: Vec<(Ident, Ident, usize)>,
+ //  struct_members: Vec<proc_macro2::TokenStream>,
  row_casters: Vec<proc_macro2::TokenStream>,
 }
 
 impl MembersAndCasters {
  fn create(members: Vec<(Ident, Ident, usize)>) -> MembersAndCasters {
-  let struct_members: Vec<_> = members.iter().map(|(name, ty, _i)| quote!(#name: #ty)).collect();
+  // let struct_members: Vec<_> = members.iter().map(|(name, ty, _i)| quote!(#name: #ty)).collect();
   let row_casters =
    members.iter().map(|(name, _ty, i)| quote!(#name: row.get(#i)?)).collect::<Vec<_>>();
 
-  Self { members, struct_members, row_casters }
+  Self { row_casters }
  }
 }
 
@@ -233,7 +199,8 @@ use ParseStatementType::{Execute, Select};
 
 #[derive(Debug)]
 struct StatementInfo {
- parameter_count: usize,
+ positional_parameter_count: usize,
+ named_parameters: Vec<String>,
  column_names: Vec<String>,
 }
 
@@ -281,7 +248,7 @@ fn migrations_to_schema(migrations: &[String]) -> Result<String, rusqlite::Error
  Ok(
   migrations_to_tempdb(migrations)
    .prepare("SELECT sql FROM sqlite_master WHERE type='table' ORDER BY sql")?
-   .query_map(params![], |row| Ok(row.get(0)?))?
+   .query_map(params![], |row| row.get(0))?
    .collect::<Result<Vec<String>, _>>()?
    .join("\n"),
  )
@@ -291,7 +258,7 @@ fn read_migrations_toml() -> MigrationsToml {
  let lockfile = std::fs::File::create(std::env::temp_dir().join("migrations.toml.lock")).unwrap();
  fs2::FileExt::lock_exclusive(&lockfile).unwrap();
 
- let migrations_toml_path = std::env::current_dir().unwrap().join(MIGRATIONS_FILENAME);
+ let migrations_toml_path = migrations_toml_path();
  let migrations_toml_path_lossy = migrations_toml_path.to_string_lossy();
 
  match migrations_toml_path.exists() {
@@ -312,14 +279,20 @@ fn read_migrations_toml() -> MigrationsToml {
 fn validate_sql<S: AsRef<str>>(sql: S) -> Result<StatementInfo, rusqlite::Error> {
  let tempdb = migrations_to_tempdb(&read_migrations_toml().migrations_append_only.unwrap());
 
- let stmt = tempdb.prepare(sql.as_ref());
+ let stmt = tempdb.prepare(sql.as_ref())?;
+ let mut positional_parameter_count = stmt.parameter_count();
+ let mut named_parameters = Vec::new();
 
- // eprintln!("{:#?}", stmt);
-
- let stmt = stmt?;
+ for idx in 1..=stmt.parameter_count() {
+  if let Some(parameter_name) = stmt.parameter_name(idx) {
+   named_parameters.push(parameter_name.to_string());
+   positional_parameter_count -= 1;
+  }
+ }
 
  Ok(StatementInfo {
-  parameter_count: stmt.parameter_count(),
+  positional_parameter_count,
+  named_parameters,
   column_names: stmt.column_names().into_iter().map(str::to_string).collect(),
  })
 }
@@ -330,6 +303,39 @@ fn validate_sql_or_abort<S: AsRef<str> + std::fmt::Debug>(sql: S) -> StatementIn
  })
 }
 
+fn parse_interpolated_sql(
+ input: ParseStream,
+) -> syn::Result<(Option<String>, Punctuated<Expr, Token![,]>)> {
+ let mut params: Punctuated<Expr, Token![,]> = Punctuated::new();
+
+ if input.is_empty() {
+  return Ok((None, params));
+ }
+
+ let mut sql = input.parse::<LitStr>()?.value();
+
+ if input.parse::<Token![,]>().is_ok() {
+  return Ok((Some(sql), input.parse_terminated(Expr::parse)?));
+ }
+
+ loop {
+  if input.is_empty() {
+   break;
+  }
+
+  params.push(input.parse()?);
+  sql.push('?');
+
+  if input.is_empty() {
+   break;
+  }
+
+  sql.push_str(&input.parse::<LitStr>()?.value());
+ }
+
+ Ok((Some(sql), params))
+}
+
 fn do_parse_tokens(
  input: ParseStream,
  statement_type: ParseStatementType,
@@ -339,7 +345,15 @@ fn do_parse_tokens(
  // Get result type and SQL
 
  let result_type = input.parse::<Type>().ok();
- let sql = input.parse::<LitStr>().ok().map(|s| s.value());
+ let (sql, params) = parse_interpolated_sql(input)?;
+
+ if std::env::current_exe().unwrap().file_stem().unwrap() == "rust-analyzer" {
+  if let Some(ty) = result_type {
+   return Ok(quote!(Ok(#ty::default())));
+  } else {
+   return Ok(quote!());
+  }
+ }
 
  // Try validating SQL as-is
 
@@ -409,28 +423,30 @@ fn do_parse_tokens(
   (Some(ResultType { contents: Some(contents), .. }), sql, None) => {
    let result_type = contents.to_string();
    let table_name = result_type.to_lowercase();
-   let tables = TABLES.lock().unwrap();
-   let table = match tables.get(&table_name) {
-    Some(t) => t.clone(),
-    None => {
-     let t = match read_migrations_toml().output_generated_tables_do_not_edit {
-      Some(m) => m.get(&table_name).cloned(),
-      None => None,
-     };
+   // let tables = TABLES.lock().unwrap();
+   // let table = match tables.get(&table_name) {
+   //  Some(t) => t.clone(),
+   //  None =>
 
-     match t {
-      Some(t) => t,
-      None => {
-       abort!(
-        span,
-        "Table {:?} not found. Does struct {} exist and have #[derive(Turbosql, Default)]?",
-        table_name,
-        result_type
-       );
-      }
+   let table = {
+    let t = match read_migrations_toml().output_generated_tables_do_not_edit {
+     Some(m) => m.get(&table_name).cloned(),
+     None => None,
+    };
+
+    match t {
+     Some(t) => t,
+     None => {
+      abort!(
+       span,
+       "Table {:?} not found. Does struct {} exist and have #[derive(Turbosql, Default)]?",
+       table_name,
+       result_type
+      );
      }
     }
    };
+   // };
 
    let column_names_str =
     table.columns.iter().map(|c| c.name.as_str()).collect::<Vec<_>>().join(", ");
@@ -458,14 +474,18 @@ fn do_parse_tokens(
 
  // get query params and validate their count against what the statement is expecting
 
- let QueryParams { params } = input.parse()?;
+ if !stmt_info.named_parameters.is_empty() {
+  abort_call_site!("SQLite named parameters not currently supported.");
+ }
 
- if params.len() != stmt_info.parameter_count {
-  abort!(
-   span,
+ if params.len() != stmt_info.positional_parameter_count {
+  #[cfg(test)]
+  todo_or_die::issue_closed!("rust-lang", "rust", 54725); // span.join()
+
+  abort_call_site!(
    "Expected {} bound parameter{}, got {}: {:?}",
-   stmt_info.parameter_count,
-   if stmt_info.parameter_count == 1 { "" } else { "s" },
+   stmt_info.positional_parameter_count,
+   if stmt_info.positional_parameter_count == 1 { "" } else { "s" },
    params.len(),
    sql
   );
@@ -475,6 +495,16 @@ fn do_parse_tokens(
   return Err(input.error("Expected parameters"));
  }
 
+ let params = if stmt_info.named_parameters.is_empty() {
+  quote! { ::turbosql::params![#params] }
+ } else {
+  let param_quotes = stmt_info.named_parameters.iter().map(|p| {
+   let var_ident = format_ident!("{}", &p[1..]);
+   quote!(#p: &#var_ident,)
+  });
+  quote! { ::turbosql::named_params![#(#param_quotes),*] }
+ };
+
  // if we return no columns, this should be an execute
 
  if stmt_info.column_names.is_empty() {
@@ -488,7 +518,7 @@ fn do_parse_tokens(
     ::turbosql::__TURBOSQL_DB.with(|db| {
      let db = db.borrow_mut();
      let mut stmt = db.prepare_cached(#sql)?;
-     stmt.execute(::turbosql::params![#params])
+     stmt.execute(#params)
     })
    })()
   }
@@ -553,7 +583,7 @@ fn do_parse_tokens(
       ::turbosql::__TURBOSQL_DB.with(|db| {
        let db = db.borrow_mut();
        let mut stmt = db.prepare_cached(#sql)?;
-       let result = stmt.query_map(::turbosql::params![#params], |row| {
+       let result = stmt.query_map(#params, |row| {
         Ok(#contents {
          #(#row_casters),*
          // #default
@@ -586,7 +616,7 @@ fn do_parse_tokens(
       ::turbosql::__TURBOSQL_DB.with(|db| {
        let db = db.borrow_mut();
        let mut stmt = db.prepare_cached(#sql)?;
-       let result = stmt.query_row(::turbosql::params![#params], |row| -> ::turbosql::Result<#contents> {
+       let result = stmt.query_row(#params, |row| -> ::turbosql::Result<#contents> {
         Ok(#contents {
          #(#row_casters),*
          // #default
@@ -603,7 +633,7 @@ fn do_parse_tokens(
   // Primitive type
   Some(ResultType { container: None, contents: Some(contents) })
    if ["f32", "f64", "i8", "u8", "i16", "u16", "i32", "u32", "i64", "String", "bool"]
-    .contains(&&contents.to_string().as_str()) =>
+    .contains(&contents.to_string().as_str()) =>
   {
    quote! {
     {
@@ -611,8 +641,8 @@ fn do_parse_tokens(
       ::turbosql::__TURBOSQL_DB.with(|db| {
        let db = db.borrow_mut();
        let mut stmt = db.prepare_cached(#sql)?;
-       let result = stmt.query_row(::turbosql::params![#params], |row| -> ::turbosql::Result<#contents> {
-        Ok(row.get(0)?)
+       let result = stmt.query_row(#params, |row| -> ::turbosql::Result<#contents> {
+        row.get(0)
        })?;
        Ok(result)
       })
@@ -634,7 +664,7 @@ fn do_parse_tokens(
       ::turbosql::__TURBOSQL_DB.with(|db| {
        let db = db.borrow_mut();
        let mut stmt = db.prepare_cached(#sql)?;
-       let result = stmt.query_row(::turbosql::params![#params], |row| -> ::turbosql::Result<#contents> {
+       let result = stmt.query_row(#params, |row| -> ::turbosql::Result<#contents> {
         Ok(#contents {
          #(#row_casters),*
          // #default
@@ -687,6 +717,10 @@ pub fn select(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
 #[proc_macro_derive(Turbosql, attributes(turbosql))]
 #[proc_macro_error]
 pub fn turbosql_derive_macro(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
+ if std::env::current_exe().unwrap().file_stem().unwrap() == "rust-analyzer" {
+  return quote!().into();
+ }
+
  // parse tokenstream and set up table struct
 
  let input = parse_macro_input!(input as DeriveInput);
@@ -694,11 +728,6 @@ pub fn turbosql_derive_macro(input: proc_macro::TokenStream) -> proc_macro::Toke
  let table_ident = input.ident;
  let table_name = table_ident.to_string().to_lowercase();
 
- let ltn = LAST_TABLE_NAME.lock().unwrap().clone();
-
- let mut last_table_name_ref = LAST_TABLE_NAME.lock().unwrap();
- *last_table_name_ref = format!("{}, {}", ltn, table_name);
-
  let fields = match input.data {
   Data::Struct(ref data) => match data.fields {
    Fields::Named(ref fields) => fields,
@@ -715,7 +744,7 @@ pub fn turbosql_derive_macro(input: proc_macro::TokenStream) -> proc_macro::Toke
  };
 
  let minitable = MiniTable {
-  name: table_name.clone(),
+  name: table_name,
   columns: table
    .columns
    .iter()
@@ -727,8 +756,7 @@ pub fn turbosql_derive_macro(input: proc_macro::TokenStream) -> proc_macro::Toke
    .collect(),
  };
 
- TABLES.lock().unwrap().insert(table_name, minitable);
- create(&table);
+ create(&table, &minitable);
 
  // create trait functions
 
@@ -737,13 +765,15 @@ pub fn turbosql_derive_macro(input: proc_macro::TokenStream) -> proc_macro::Toke
 
  // output tokenstream
 
- proc_macro::TokenStream::from(quote! {
+ let output = quote! {
   #[cfg(not(target_arch = "wasm32"))]
-  impl #table {
+  impl ::turbosql::Turbosql for #table {
    #fn_insert
    #fn_update
   }
- })
+ };
+
+ output.into()
 }
 
 /// Convert syn::FieldsNamed to our Column type.
@@ -785,7 +815,10 @@ fn extract_columns(fields: &FieldsNamed) -> Vec<Column> {
    // specifically, sqlite cannot represent u64 integers, would be coerced to float.
    // https://sqlite.org/fileformat.html
 
-   let sql_type = match (name.as_str(), ty_str.as_str()) {
+   let sql_type = match (
+    name.as_str(),
+    if U8_ARRAY_RE.is_match(&ty_str) { "Option < [u8; _] >" } else { ty_str.as_str() },
+   ) {
     ("rowid", "Option < i64 >") => "INTEGER PRIMARY KEY",
     // (_, "i64") => "INTEGER NOT NULL",
     (_, "Option < i8 >") => "INTEGER",
@@ -795,7 +828,7 @@ fn extract_columns(fields: &FieldsNamed) -> Vec<Column> {
     (_, "Option < i32 >") => "INTEGER",
     (_, "Option < u32 >") => "INTEGER",
     (_, "Option < i64 >") => "INTEGER",
-    (_, "u64") => abort!(ty, SQLITE_U64_ERROR),
+    // (_, "u64") => abort!(ty, SQLITE_U64_ERROR),
     (_, "Option < u64 >") => abort!(ty, SQLITE_U64_ERROR),
     // (_, "f64") => "REAL NOT NULL",
     (_, "Option < f64 >") => "REAL",
@@ -807,7 +840,19 @@ fn extract_columns(fields: &FieldsNamed) -> Vec<Column> {
     // SELECT LENGTH(blob_column) ... will be null if blob is null
     // (_, "Blob") => "BLOB NOT NULL",
     (_, "Option < Blob >") => "BLOB",
-    _ => abort!(ty, "turbosql doesn't support rust type: {}", ty_str),
+    (_, "Option < Vec < u8 > >") => "BLOB",
+    (_, "Option < [u8; _] >") => "BLOB",
+    _ => {
+     if ty_str.starts_with("Option < ") {
+      abort!(ty, "Turbosql doesn't support rust type: {}", ty_str)
+     } else {
+      abort!(
+       ty,
+       "Turbosql types must be wrapped in Option for forward/backward schema compatibility. Try: Option<{}>",
+       ty_str
+      )
+     }
+    }
    };
 
    Some(Column {
@@ -837,23 +882,23 @@ fn extract_columns(fields: &FieldsNamed) -> Vec<Column> {
 use std::fs;
 
 /// CREATE TABLE
-fn create(table: &Table) {
+fn create(table: &Table, minitable: &MiniTable) {
  // create the migrations
 
- let sql = makesql_create(&table);
+ let sql = makesql_create(table);
 
  rusqlite::Connection::open_in_memory().unwrap().execute(&sql, params![]).unwrap_or_else(|e| {
   abort_call_site!("Error validating auto-generated CREATE TABLE statement: {} {:#?}", sql, e)
  });
 
- let target_migrations = make_migrations(&table);
+ let target_migrations = make_migrations(table);
 
  // read in the existing migrations from toml
 
  let lockfile = std::fs::File::create(std::env::temp_dir().join("migrations.toml.lock")).unwrap();
  fs2::FileExt::lock_exclusive(&lockfile).unwrap();
 
- let migrations_toml_path = std::env::current_dir().unwrap().join(MIGRATIONS_FILENAME);
+ let migrations_toml_path = migrations_toml_path();
  let migrations_toml_path_lossy = migrations_toml_path.to_string_lossy();
 
  let old_toml_str = if migrations_toml_path.exists() {
@@ -871,6 +916,7 @@ fn create(table: &Table) {
 
  let mut output_migrations = source_migrations_toml.migrations_append_only.unwrap_or_default();
 
+ #[allow(clippy::search_is_some)]
  target_migrations.iter().for_each(|target_m| {
   if output_migrations
    .iter()
@@ -881,16 +927,8 @@ fn create(table: &Table) {
   }
  });
 
- let tables = match source_migrations_toml.output_generated_tables_do_not_edit {
-  Some(ref t) => {
-   let mut t = t.clone();
-   TABLES.lock().unwrap().iter().for_each(|(k, v)| {
-    t.insert(k.clone(), v.clone());
-   });
-   t
-  }
-  None => TABLES.lock().unwrap().clone(),
- };
+ let mut tables = source_migrations_toml.output_generated_tables_do_not_edit.unwrap_or_default();
+ tables.insert(table.name.clone(), minitable.clone());
 
  // save to toml
 
@@ -903,10 +941,10 @@ fn create(table: &Table) {
    "  {}\n",
    migrations_to_schema(&output_migrations)
     .unwrap()
-    .replace("\n", "\n  ")
-    .replace("(", "(\n    ")
+    .replace('\n', "\n  ")
+    .replace('(', "(\n    ")
     .replace(", ", ",\n    ")
-    .replace(")", "\n  )")
+    .replace(')', "\n  )")
   )),
   migrations_append_only: Some(output_migrations),
   output_generated_tables_do_not_edit: Some(tables),
@@ -914,13 +952,7 @@ fn create(table: &Table) {
  .serialize(&mut serializer)
  .unwrap_or_else(|e| abort_call_site!("Unable to serialize migrations toml: {:?}", e));
 
- let new_toml_str = indoc::formatdoc! {"
-  # This file is auto-generated by Turbosql.
-  # It is used to create and apply automatic schema migrations.
-  # It should be checked into source control.
-  # Modifying it by hand may be dangerous; see the docs.
-
-  {}", &new_toml_str};
+ let new_toml_str = format!("# This file is auto-generated by Turbosql.\n# It is used to create and apply automatic schema migrations.\n# It should be checked into source control.\n# Modifying it by hand may be dangerous; see the docs.\n\n{}", &new_toml_str);
 
  // Only write migrations.toml file if it has actually changed;
  // this keeps file mod date clean so cargo doesn't pathologically rebuild
@@ -955,3 +987,13 @@ fn make_migrations(table: &Table) -> Vec<String> {
 
  vec
 }
+
+fn migrations_toml_path() -> std::path::PathBuf {
+ let mut path = std::path::PathBuf::from(env!("OUT_DIR"));
+ while path.file_name() != Some(std::ffi::OsStr::new("target")) {
+  path.pop();
+ }
+ path.pop();
+ path.push(MIGRATIONS_FILENAME);
+ path
+}
